@@ -9,6 +9,7 @@ from collections import Counter
 import random
 import math
 import warnings
+import copy
 
 import numpy as np
 
@@ -926,7 +927,7 @@ class CharPostprocessor(BaseDataPostprocessor,
         return output_result
 
     @staticmethod
-    def match_sentence_lengths(data, results, flatten_separator):
+    def match_sentence_lengths(data, results, flatten_separator, inplace=True):
         """
         Converts the results from the model into the same ragged data shapes as
         the original data.
@@ -938,6 +939,8 @@ class CharPostprocessor(BaseDataPostprocessor,
         :param flatten_separator: string which joins to samples together when
                                   flattening
         :type flatten_separator: str
+        :param inplace: flag to modify results in place
+        :type inplace: bool
         :return: dict(pred=...) or dict(pred=..., conf=...)
         """
         pred_buffer = []
@@ -946,6 +949,9 @@ class CharPostprocessor(BaseDataPostprocessor,
         buffer_add_inds = np.cumsum(list(map(len, results['pred']))).tolist()
         separator_len = len(flatten_separator)
 
+        if not inplace:
+            results = copy.deepcopy(results) 
+
         if results['pred']:
             pred_buffer = np.concatenate(results['pred'])
         results['pred'] = [np.array([])] * len(data)
@@ -1328,7 +1334,7 @@ class StructCharPostprocessor(BaseDataPostprocessor,
         print(help_str)
 
     @staticmethod
-    def match_sentence_lengths(data, results, flatten_separator):
+    def match_sentence_lengths(data, results, flatten_separator, inplace=True):
         """
         Converts the results from the model into the same ragged data shapes as
         the original data.
@@ -1340,6 +1346,8 @@ class StructCharPostprocessor(BaseDataPostprocessor,
         :param flatten_separator: string which joins to samples together when
                                   flattening
         :type flatten_separator: str
+        :param inplace: flag to modify results in place
+        :type inplace: bool
         :return: dict(pred=...) or dict(pred=..., conf=...)
         """
         pred_buffer = []
@@ -1348,6 +1356,9 @@ class StructCharPostprocessor(BaseDataPostprocessor,
         buffer_add_inds = np.cumsum(list(map(len, results['pred']))).tolist()
         separator_len = len(flatten_separator)
 
+        if not inplace:
+            results = copy.deepcopy(results) 
+
         if results['pred']:
             pred_buffer = np.concatenate(results['pred'])
         results['pred'] = [np.array([])] * len(data)
