@@ -1,4 +1,6 @@
 import re
+import copy
+import math
 import numpy as np
 
 from .numerical_column_stats import NumericStatsMixin
@@ -28,7 +30,26 @@ class FloatColumn(NumericStatsMixin, BaseColumnPrimitiveTypeProfiler):
                              " FloatOptions.")
         NumericStatsMixin.__init__(self, options)
         BaseColumnPrimitiveTypeProfiler.__init__(self, name)
-        self.precision = 0
+
+        self.precision = {
+            'min': None,
+            'max': None,
+            'mean': None,
+            'var': None,
+            'std': None,
+            'sum': None,
+            'sample_size': None,
+            'margin_of_error': None,
+            'confidence_level': 0.999
+        }
+        
+        # https://www.calculator.net/confidence-interval-calculator.html
+        self.__z_value_precision = 3.291
+
+        self.__precision_sample_ratio = None
+        if options and options.precision and options.precision.is_enabled:
+            self.__precision_sample_ratio = options.precision.sample_ratio
+        
         self.__calculations = {
             "precision": FloatColumn._update_precision,
         }
@@ -57,7 +78,39 @@ class FloatColumn(NumericStatsMixin, BaseColumnPrimitiveTypeProfiler):
                                  other.__calculations)
 
         if "precision" in merged_profile.__calculations:
-            merged_profile.precision = min(self.precision, other.precision)
+
+            if self.precision['min'] is None:
+                merged_profile.precision = copy.deepcopy(other.precision)
+            elif other.precision['min'] is None:
+                merged_profile.precision = copy.deepcopy(self.precision)                
+            else:
+                merged_profile.precision['min'] = min(
+                    self.precision['min'], other.precision['min'])
+                merged_profile.precision['max'] = max(
+                    self.precision['max'], other.precision['max'])
+                merged_profile.precision['sum'] = \
+                    self.precision['sum'] + other.precision['sum']
+                merged_profile.precision['sample_size'] = \
+                    self.precision['sample_size'] + other.precision['sample_size']
+
+                merged_profile.precision['var'] = self._merge_variance(
+                    self.precision['sample_size'],
+                    self.precision['var'], self.precision['mean'],
+                    other.precision['sample_size'], other.precision['var'],
+                    other.precision['mean'])
+                merged_profile.precision['mean'] = \
+                    merged_profile.precision['sum'] \
+                    / merged_profile.precision['sample_size']
+            
+                merged_profile.precision['std'] = math.sqrt(
+                    merged_profile.precision['var'])
+
+                # Margin of error, 99.9% confidence level
+                merged_profile.precision['margin_of_error'] = \
+                    merged_profile.__z_value_precision * (
+                        merged_profile.precision['std']
+                        / math.sqrt(merged_profile.precision['sample_size'])
+                )
             
         return merged_profile
 
@@ -67,21 +120,28 @@ class FloatColumn(NumericStatsMixin, BaseColumnPrimitiveTypeProfiler):
         Property for profile. Returns the profile of the column.
         :return:
         """
-        histogram_method = self.histogram_bin_method_names[0]
-        if self.histogram_selection is not None:
-            histogram_method = self.histogram_selection
 
         profile = dict(
-            min=self.min,
-            max=self.max,
-            mean=self.mean,
-            variance=self.variance,
-            stddev=self.stddev,
-            histogram=self.histogram_methods[histogram_method]['histogram'],
+            min=self.np_type_to_type(self.min),
+            max=self.np_type_to_type(self.max),
+            mean=self.np_type_to_type(self.mean),
+            variance=self.np_type_to_type(self.variance),
+            stddev=self.np_type_to_type(self.stddev),
+            histogram=self._get_best_histogram_for_profile(),
             quantiles=self.quantiles,
             times=self.times,
-            precision=self.precision
+            precision=dict(
+                min=self.np_type_to_type(self.precision['min']),
+                max=self.np_type_to_type(self.precision['max']),
+                mean=self.np_type_to_type(self.precision['mean']),
+                var=self.np_type_to_type(self.precision['var']),
+                std=self.np_type_to_type(self.precision['std']),
+                sample_size=self.np_type_to_type(self.precision['sample_size']),
+                margin_of_error=self.np_type_to_type(self.precision['margin_of_error']),
+                confidence_level=self.np_type_to_type(self.precision['confidence_level'])
+            )
         )
+        
         return profile
 
     @property
@@ -96,27 +156,49 @@ class FloatColumn(NumericStatsMixin, BaseColumnPrimitiveTypeProfiler):
         return None
 
     @classmethod
-    def _get_float_precision(cls, df_series_clean):
+    def _get_float_precision(cls, df_series_clean, sample_ratio=None):
         """
         Determines the precision of the numeric value.
         
         :param df_series_clean: df series with nulls removed, assumes all values
             are floats as well
         :type df_series_clean: pandas.core.series.Series
+        :param sample_ratio: Ratio of samples used for float precision
+        :type sample_ratio: float (between 0 and 1)
         :return: string representing its precision print format
         :rtype: int
         """
-        if not len(df_series_clean):
-            return 0
+        len_df = len(df_series_clean)
+        if not len_df: return None
 
         # Lead zeros: ^[+-.0\s]+ End zeros: \.?0+(\s|$)
         # Scientific Notation: (?<=[e])(.*) Any non-digits: \D
         r = re.compile(r'^[+-.0\s]+|\.?0+(\s|$)|(?<=[e])(.*)|\D')
-        
-        return float(df_series_clean.replace(
-            to_replace=r, value='').map(len).min())
-        
 
+        # DEFAULT: Sample the dataset. If small use full dataset,
+        # OR 20k samples or 5% of the dataset which ever is larger.
+        # If user sets sample ratio, utilize their request
+        sample_size = min(len_df, max(20000, int(len_df * 0.05)))
+        if sample_ratio is not None and sample_ratio > 0:
+            sample_size = int(len_df * sample_ratio)
+
+        # length of sampled cells after all punctuation removed
+        len_per_float = df_series_clean.sample(sample_size).replace(
+            to_replace=r, value='').map(len)
+
+        # Determine statistics precision
+        precision_sum = len_per_float.sum()
+        subset_precision = {
+            'min': len_per_float.min(),
+            'max': len_per_float.max(),
+            'mean': precision_sum / sample_size,
+            'var': float(len_per_float.var()),
+            'sum': precision_sum,
+            'sample_size': sample_size
+        }
+        
+        return subset_precision
+    
     @classmethod
     def _is_each_row_float(cls, df_series):
         """
@@ -131,11 +213,8 @@ class FloatColumn(NumericStatsMixin, BaseColumnPrimitiveTypeProfiler):
         :return: is_float_col
         :rtype: list
         """
-        len_df = len(df_series)
-        if len_df == 0:
-            return list()
-
-        return [NumericStatsMixin.is_float(x) for x in df_series]
+        if len(df_series) == 0: return list()
+        return df_series.map(NumericStatsMixin.is_float)
 
     @BaseColumnProfiler._timeit(name='precision')
     def _update_precision(self, df_series, prev_dependent_properties,
@@ -153,12 +232,48 @@ class FloatColumn(NumericStatsMixin, BaseColumnPrimitiveTypeProfiler):
         :type df_series: pandas.DataFrame
         :return: None
         """
-        subset_precision = self._get_float_precision(df_series)
-        if self.precision and subset_precision:
-            self.precision = min(self.precision, subset_precision)
-        elif subset_precision:
-            self.precision = subset_precision
 
+        sample_ratio = None
+        if self.__precision_sample_ratio is not None:
+            sample_ratio = self.__precision_sample_ratio
+        
+        # (min, max, var, sum, sample_size)
+        subset_precision = self._get_float_precision(df_series, sample_ratio)
+        if subset_precision is None:
+            return
+        elif self.precision['min'] is None:
+            self.precision.update(subset_precision)
+        else:        
+            # Update the calculations as data is valid
+            self.precision['min'] = min(
+                self.precision['min'], subset_precision['min'])
+            self.precision['max'] = max(
+                self.precision['max'], subset_precision['max'])            
+            self.precision['sum'] += subset_precision['sum']
+            
+            self.precision['var'] = self._merge_variance(
+                self.precision['sample_size'], self.precision['var'],
+                self.precision['mean'],
+                subset_precision['sample_size'], subset_precision['var'],
+                subset_precision['mean'])
+            
+            self.precision['sample_size'] += subset_precision['sample_size']            
+            self.precision['mean'] = self.precision['sum'] \
+                / self.precision['sample_size']
+
+        # Calculated outside
+        self.precision['std'] = math.sqrt(self.precision['var'])
+
+        # Margin of error, 99.9% confidence level
+        self.precision['margin_of_error'] = self.__z_value_precision *(
+            self.precision['std'] / math.sqrt(self.precision['sample_size']))
+
+        # Set the significant figures
+        sigfigs = int(self.precision['max'])
+        for key in ['mean', 'var', 'std', 'margin_of_error']:
+            self.precision[key] = \
+                float('{:.{p}g}'.format(self.precision[key], p=sigfigs))
+                        
     def _update_helper(self, df_series_clean, profile):
         """
         Method for updating the column profile properties with a cleaned
@@ -200,7 +315,6 @@ class FloatColumn(NumericStatsMixin, BaseColumnPrimitiveTypeProfiler):
         if len(df_series) == 0:
             return self
         
-        df_series = df_series.reset_index(drop=True)
         is_each_row_float = self._is_each_row_float(df_series)
         sample_size = len(is_each_row_float)
         float_count = np.sum(is_each_row_float)
@@ -216,4 +330,3 @@ class FloatColumn(NumericStatsMixin, BaseColumnPrimitiveTypeProfiler):
         )
 
         return self
-        
