@@ -17,13 +17,13 @@ class TestCSVDataClass(unittest.TestCase):
         test_dir = os.path.join(test_root_path, 'data')
         cls.input_file_names = [
             dict(path=os.path.join(test_dir, 'csv/diamonds.csv'),
-                 count=1000, delimiter=',', has_header=[0],
+                 count=53940, delimiter=',', has_header=[0],
                  num_columns=10, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/iris.csv'),
                  count=150, delimiter=',', has_header=[0],
                  num_columns=6, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/iris_no_header.csv'),
-                 count=150, delimiter=',', has_header=[None],
+                 count=20, delimiter=',', has_header=[None],
                  num_columns=6, encoding='utf-8'),            
             dict(path=os.path.join(test_dir, 'csv/iris-utf-8.csv'),
                  count=150, delimiter=',', has_header=[0],
@@ -41,13 +41,13 @@ class TestCSVDataClass(unittest.TestCase):
                  count=150, delimiter=',', has_header=[0],
                  num_columns=6, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/guns.csv'),
-                 count=1316, delimiter=',', has_header=[0],
+                 count=100798, delimiter=',', has_header=[0],
                  num_columns=10, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/wisconsin_cancer_train.csv'),
-                 count=25, delimiter=',', has_header=[0],
+                 count=497, delimiter=',', has_header=[0],
                  num_columns=10, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/aws_honeypot_marx_geo.csv'),
-                 count=25, delimiter=',', has_header=[0],
+                 count=2999, delimiter=',', has_header=[0],
                  num_columns=16, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/small-num.csv'),
                  count=5, delimiter=None, has_header=[0],
@@ -56,25 +56,25 @@ class TestCSVDataClass(unittest.TestCase):
                  count=5, delimiter=None, has_header=[None],
                  num_columns=1, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/names-col.txt'),
-                 count=5, delimiter=None, has_header=[0],
+                 count=6, delimiter=None, has_header=[0],
                  num_columns=1, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/names-col-empty.txt'),
-                 count=5, delimiter=None, has_header=[0],
+                 count=6, delimiter=None, has_header=[0],
                  num_columns=1, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/log_data_long.txt'),
                  count=753, delimiter=',', has_header=[None],
                  num_columns=3, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/sparse-last-column.txt'),
-                 count=5, delimiter=',', has_header=[0],
+                 count=9, delimiter=',', has_header=[0],
                  num_columns=2, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/sparse-first-column.txt'),
-                 count=5, delimiter=',', has_header=[0],
+                 count=9, delimiter=',', has_header=[0],
                  num_columns=2, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/sparse-first-and-last-column.txt'),
-                 count=5, delimiter=',', has_header=[0],
+                 count=9, delimiter=',', has_header=[0],
                  num_columns=3, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/sparse-first-and-last-column-no-header.txt'),
-                 count=5, delimiter=',', has_header=[None],
+                 count=9, delimiter=',', has_header=[None],
                  num_columns=3, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/log_data_sparse.txt'),
                  count=20, delimiter=',', has_header=[None],
@@ -83,49 +83,49 @@ class TestCSVDataClass(unittest.TestCase):
                  count=20, delimiter=',', has_header=[None],
                  num_columns=6, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/sparse-columns-test.csv'),
-                 count=20, delimiter=',', has_header=[0],
+                 count=25, delimiter=',', has_header=[0],
                  num_columns=36, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/sentence-4x.txt'),
                  count=4, delimiter='.', has_header=[0, None],
                  num_columns=1, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/quote-test.txt'),
-                 count=10, delimiter=' ', has_header=[0, None],
+                 count=8, delimiter=' ', has_header=[0, None],
                  num_columns=3, encoding='utf-8'),            
             dict(path=os.path.join(test_dir, 'csv/quote-test-singlequote.txt'),
-                 count=9, delimiter=' ', has_header=[0, None],
+                 count=8, delimiter=' ', has_header=[0, None],
                  num_columns=3, encoding='utf-8'),            
             dict(path=os.path.join(test_dir, 'csv/multiple-col-delimiter-last.txt'),
-                 count=6, delimiter=',', has_header=[0],
+                 count=5, delimiter=',', has_header=[0],
                  num_columns=4, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/names-col-without-space.txt'),
-                 count=6, delimiter=None, has_header=[0],
+                 count=10, delimiter=None, has_header=[0],
                  num_columns=1, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/sparse-first-and-last-column-two-headers.txt'),
-                 count=6, delimiter=',', has_header=[1],
+                 count=9, delimiter=',', has_header=[1],
                  num_columns=3, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/sparse-interchange-none.txt'),
-                 count=6, delimiter=',', has_header=[0],
+                 count=9, delimiter=',', has_header=[0],
                  num_columns=3, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/blogposts.csv'),
                  count=25, delimiter=',', has_header=[0],
                  num_columns=4, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/all-strings-standard-header.csv'),
-                 count=11, delimiter=',', has_header=[0],
+                 count=10, delimiter=',', has_header=[0],
                  num_columns=4, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/all-strings-standard-header-quotes.csv'),
-                 count=11, delimiter=',', has_header=[0],
+                 count=10, delimiter=',', has_header=[0],
                  num_columns=4, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/all-strings-standard-header-data-quotes.csv'),
-                 count=11, delimiter=',', has_header=[0],
+                 count=10, delimiter=',', has_header=[0],
                  num_columns=4, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/all-strings-skip-header.csv'),
-                 count=11, delimiter=',', has_header=[1],
+                 count=9, delimiter=',', has_header=[1],
                  num_columns=4, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/all-strings-skip-header-author.csv'),
-                count=7, delimiter=',', has_header=[1],
+                 count=5, delimiter=',', has_header=[1],
                  num_columns=4, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/all-strings-skip-partial-header.csv'),
-                 count=7, delimiter=',', has_header=[None, 1],
+                 count=6, delimiter=',', has_header=[None, 1],
                  num_columns=4, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/num-negative-title.csv'),
                  count=5, delimiter=None, has_header=[None],
@@ -134,20 +134,20 @@ class TestCSVDataClass(unittest.TestCase):
                  count=5, delimiter=None, has_header=[None],
                  num_columns=1, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/daily-activity-sheet-@.csv'),
-                 count=31, delimiter='@', has_header=[1],
+                 count=30, delimiter='@', has_header=[1],
                  num_columns=4, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/daily-activity-sheet-int-description.csv'),
-                 count=32, delimiter=',', has_header=[1],
+                 count=30, delimiter=',', has_header=[1],
                  num_columns=4, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/daily-activity-sheet-@-singlequote.csv'),
-                 count=31, delimiter='@', has_header=[1],
+                 count=30, delimiter='@', has_header=[1],
                  num_columns=4, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/daily-activity-sheet-tab.csv'),
-                 count=31, delimiter='\t', has_header=[0],
+                 count=30, delimiter='\t', has_header=[0],
                  num_columns=4, encoding='utf-8'),
             dict(path=os.path.join(test_dir, 'csv/preferred-check-small-num.csv'),
                  count=5, delimiter=',', has_header=[None],
-                 num_columns=2, encoding='utf-8'),
+                 num_columns=2, encoding='utf-8')
         ]
         cls.output_file_path = None
         
@@ -338,7 +338,22 @@ class TestCSVDataClass(unittest.TestCase):
                       expected_error = "'selected_columns' must be a list") 
         
         _test_options("selected_columns", valid = [], invalid = [[0,1,2,3]],
-                      expected_error = "'selected_columns' must be a list of strings") 
+                      expected_error = "'selected_columns' must be a list of strings")
+
+    def test_len_data(self):
+        """
+        Validate that length called on CSVData is appropriately determining the
+        length value.
+        """
+
+        for input_file in self.input_file_names:
+            data = Data(input_file["path"])
+            self.assertEqual(input_file['count'],
+                             len(data),
+                             msg=input_file['path'])
+            self.assertEqual(input_file['count'],
+                             data.length,
+                             msg=input_file['path'])
 
 
 if __name__ == '__main__':
